
package org.mule.modules.workday.cash.config.spring;

import java.util.List;
import org.apache.commons.lang.StringUtils;
import org.mule.config.spring.MuleHierarchicalBeanDefinitionParserDelegate;
import org.mule.config.spring.util.SpringXMLUtils;
import org.mule.modules.workday.cash.config.SubmitPaymentElectionEnrollmentMessageProcessor;
import org.mule.util.TemplateParser;
import org.springframework.beans.MutablePropertyValues;
import org.springframework.beans.PropertyValue;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.beans.factory.config.RuntimeBeanReference;
import org.springframework.beans.factory.support.BeanDefinitionBuilder;
import org.springframework.beans.factory.support.ManagedList;
import org.springframework.beans.factory.support.ManagedMap;
import org.springframework.beans.factory.xml.BeanDefinitionParser;
import org.springframework.beans.factory.xml.ParserContext;
import org.springframework.util.xml.DomUtils;
import org.w3c.dom.Element;

public class SubmitPaymentElectionEnrollmentDefinitionParser
    implements BeanDefinitionParser
{

    /**
     * Mule Pattern Info
     * 
     */
    private TemplateParser.PatternInfo patternInfo;

    public SubmitPaymentElectionEnrollmentDefinitionParser() {
        patternInfo = TemplateParser.createMuleStyleParser().getStyle();
    }

    public BeanDefinition parse(Element element, ParserContext parserContent) {
        BeanDefinitionBuilder builder = BeanDefinitionBuilder.rootBeanDefinition(SubmitPaymentElectionEnrollmentMessageProcessor.class.getName());
        String configRef = element.getAttribute("config-ref");
        if ((configRef!= null)&&(!StringUtils.isBlank(configRef))) {
            builder.addPropertyValue("moduleObject", configRef);
        }
        Element businessProcessParametersListElement = null;
        businessProcessParametersListElement = DomUtils.getChildElementByTagName(element, "business-process-parameters");
        List<Element> businessProcessParametersListChilds = null;
        if (businessProcessParametersListElement!= null) {
            String businessProcessParametersRef = businessProcessParametersListElement.getAttribute("ref");
            if ((businessProcessParametersRef!= null)&&(!StringUtils.isBlank(businessProcessParametersRef))) {
                if ((!businessProcessParametersRef.startsWith(patternInfo.getPrefix()))&&(!businessProcessParametersRef.endsWith(patternInfo.getSuffix()))) {
                    builder.addPropertyValue("businessProcessParameters", new RuntimeBeanReference(businessProcessParametersRef));
                } else {
                    builder.addPropertyValue("businessProcessParameters", businessProcessParametersRef);
                }
            } else {
                ManagedMap businessProcessParameters = new ManagedMap();
                businessProcessParametersListChilds = DomUtils.getChildElementsByTagName(businessProcessParametersListElement, "business-process-parameter");
                if (businessProcessParametersListChilds!= null) {
                    if (businessProcessParametersListChilds.size() == 0) {
                        businessProcessParametersListChilds = DomUtils.getChildElements(businessProcessParametersListElement);
                    }
                    for (Element businessProcessParametersChild: businessProcessParametersListChilds) {
                        String businessProcessParametersValueRef = businessProcessParametersChild.getAttribute("value-ref");
                        String businessProcessParametersKeyRef = businessProcessParametersChild.getAttribute("key-ref");
                        Object valueObject = null;
                        Object keyObject = null;
                        if ((businessProcessParametersValueRef!= null)&&(!StringUtils.isBlank(businessProcessParametersValueRef))) {
                            valueObject = new RuntimeBeanReference(businessProcessParametersValueRef);
                        } else {
                            valueObject = businessProcessParametersChild.getTextContent();
                        }
                        if ((businessProcessParametersKeyRef!= null)&&(!StringUtils.isBlank(businessProcessParametersKeyRef))) {
                            keyObject = new RuntimeBeanReference(businessProcessParametersKeyRef);
                        } else {
                            keyObject = businessProcessParametersChild.getAttribute("key");
                        }
                        if ((keyObject == null)||((keyObject instanceof String)&&StringUtils.isBlank(((String) keyObject)))) {
                            keyObject = businessProcessParametersChild.getTagName();
                        }
                        businessProcessParameters.put(keyObject, valueObject);
                    }
                }
                builder.addPropertyValue("businessProcessParameters", businessProcessParameters);
            }
        }
        Element paymentElectionEnrollmentDataListElement = null;
        paymentElectionEnrollmentDataListElement = DomUtils.getChildElementByTagName(element, "payment-election-enrollment-data");
        List<Element> paymentElectionEnrollmentDataListChilds = null;
        if (paymentElectionEnrollmentDataListElement!= null) {
            String paymentElectionEnrollmentDataRef = paymentElectionEnrollmentDataListElement.getAttribute("ref");
            if ((paymentElectionEnrollmentDataRef!= null)&&(!StringUtils.isBlank(paymentElectionEnrollmentDataRef))) {
                if ((!paymentElectionEnrollmentDataRef.startsWith(patternInfo.getPrefix()))&&(!paymentElectionEnrollmentDataRef.endsWith(patternInfo.getSuffix()))) {
                    builder.addPropertyValue("paymentElectionEnrollmentData", new RuntimeBeanReference(paymentElectionEnrollmentDataRef));
                } else {
                    builder.addPropertyValue("paymentElectionEnrollmentData", paymentElectionEnrollmentDataRef);
                }
            } else {
                ManagedMap paymentElectionEnrollmentData = new ManagedMap();
                paymentElectionEnrollmentDataListChilds = DomUtils.getChildElementsByTagName(paymentElectionEnrollmentDataListElement, "payment-election-enrollment-datum");
                if (paymentElectionEnrollmentDataListChilds!= null) {
                    if (paymentElectionEnrollmentDataListChilds.size() == 0) {
                        paymentElectionEnrollmentDataListChilds = DomUtils.getChildElements(paymentElectionEnrollmentDataListElement);
                    }
                    for (Element paymentElectionEnrollmentDataChild: paymentElectionEnrollmentDataListChilds) {
                        String paymentElectionEnrollmentDataValueRef = paymentElectionEnrollmentDataChild.getAttribute("value-ref");
                        String paymentElectionEnrollmentDataKeyRef = paymentElectionEnrollmentDataChild.getAttribute("key-ref");
                        Object valueObject = null;
                        Object keyObject = null;
                        if ((paymentElectionEnrollmentDataValueRef!= null)&&(!StringUtils.isBlank(paymentElectionEnrollmentDataValueRef))) {
                            valueObject = new RuntimeBeanReference(paymentElectionEnrollmentDataValueRef);
                        } else {
                            valueObject = paymentElectionEnrollmentDataChild.getTextContent();
                        }
                        if ((paymentElectionEnrollmentDataKeyRef!= null)&&(!StringUtils.isBlank(paymentElectionEnrollmentDataKeyRef))) {
                            keyObject = new RuntimeBeanReference(paymentElectionEnrollmentDataKeyRef);
                        } else {
                            keyObject = paymentElectionEnrollmentDataChild.getAttribute("key");
                        }
                        if ((keyObject == null)||((keyObject instanceof String)&&StringUtils.isBlank(((String) keyObject)))) {
                            keyObject = paymentElectionEnrollmentDataChild.getTagName();
                        }
                        paymentElectionEnrollmentData.put(keyObject, valueObject);
                    }
                }
                builder.addPropertyValue("paymentElectionEnrollmentData", paymentElectionEnrollmentData);
            }
        }
        BeanDefinition definition = builder.getBeanDefinition();
        definition.setAttribute(MuleHierarchicalBeanDefinitionParserDelegate.MULE_NO_RECURSE, Boolean.TRUE);
        MutablePropertyValues propertyValues = parserContent.getContainingBeanDefinition().getPropertyValues();
        if (parserContent.getContainingBeanDefinition().getBeanClassName().equals("org.mule.config.spring.factories.PollingMessageSourceFactoryBean")) {
            propertyValues.addPropertyValue("messageProcessor", definition);
        } else {
            PropertyValue messageProcessors = propertyValues.getPropertyValue("messageProcessors");
            if ((messageProcessors == null)||(messageProcessors.getValue() == null)) {
                propertyValues.addPropertyValue("messageProcessors", new ManagedList());
            }
            List listMessageProcessors = ((List) propertyValues.getPropertyValue("messageProcessors").getValue());
            listMessageProcessors.add(definition);
        }
        return definition;
    }

    protected String getAttributeValue(Element element, String attributeName) {
        if (!StringUtils.isEmpty(element.getAttribute(attributeName))) {
            return element.getAttribute(attributeName);
        }
        return null;
    }

    private String generateChildBeanName(Element element) {
        String id = SpringXMLUtils.getNameOrId(element);
        if (StringUtils.isBlank(id)) {
            String parentId = SpringXMLUtils.getNameOrId(((Element) element.getParentNode()));
            return ((("."+ parentId)+":")+ element.getLocalName());
        } else {
            return id;
        }
    }

}
