
package org.mule.modules.workday.staffing.config.spring;

import java.util.List;
import org.apache.commons.lang.StringUtils;
import org.mule.config.spring.MuleHierarchicalBeanDefinitionParserDelegate;
import org.mule.config.spring.util.SpringXMLUtils;
import org.mule.modules.workday.staffing.config.HireEmployeeMessageProcessor;
import org.mule.util.TemplateParser;
import org.springframework.beans.MutablePropertyValues;
import org.springframework.beans.PropertyValue;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.beans.factory.config.RuntimeBeanReference;
import org.springframework.beans.factory.support.BeanDefinitionBuilder;
import org.springframework.beans.factory.support.ManagedList;
import org.springframework.beans.factory.support.ManagedMap;
import org.springframework.beans.factory.xml.BeanDefinitionParser;
import org.springframework.beans.factory.xml.ParserContext;
import org.springframework.util.xml.DomUtils;
import org.w3c.dom.Element;

public class HireEmployeeDefinitionParser
    implements BeanDefinitionParser
{

    /**
     * Mule Pattern Info
     * 
     */
    private TemplateParser.PatternInfo patternInfo;

    public HireEmployeeDefinitionParser() {
        patternInfo = TemplateParser.createMuleStyleParser().getStyle();
    }

    public BeanDefinition parse(Element element, ParserContext parserContent) {
        BeanDefinitionBuilder builder = BeanDefinitionBuilder.rootBeanDefinition(HireEmployeeMessageProcessor.class.getName());
        String configRef = element.getAttribute("config-ref");
        if ((configRef!= null)&&(!StringUtils.isBlank(configRef))) {
            builder.addPropertyValue("moduleObject", configRef);
        }
        Element hireEmployeeDataListElement = null;
        hireEmployeeDataListElement = DomUtils.getChildElementByTagName(element, "hire-employee-data");
        List<Element> hireEmployeeDataListChilds = null;
        if (hireEmployeeDataListElement!= null) {
            String hireEmployeeDataRef = hireEmployeeDataListElement.getAttribute("ref");
            if ((hireEmployeeDataRef!= null)&&(!StringUtils.isBlank(hireEmployeeDataRef))) {
                if ((!hireEmployeeDataRef.startsWith(patternInfo.getPrefix()))&&(!hireEmployeeDataRef.endsWith(patternInfo.getSuffix()))) {
                    builder.addPropertyValue("hireEmployeeData", new RuntimeBeanReference(hireEmployeeDataRef));
                } else {
                    builder.addPropertyValue("hireEmployeeData", hireEmployeeDataRef);
                }
            } else {
                ManagedMap hireEmployeeData = new ManagedMap();
                hireEmployeeDataListChilds = DomUtils.getChildElementsByTagName(hireEmployeeDataListElement, "hire-employee-datum");
                if (hireEmployeeDataListChilds!= null) {
                    if (hireEmployeeDataListChilds.size() == 0) {
                        hireEmployeeDataListChilds = DomUtils.getChildElements(hireEmployeeDataListElement);
                    }
                    for (Element hireEmployeeDataChild: hireEmployeeDataListChilds) {
                        String hireEmployeeDataValueRef = hireEmployeeDataChild.getAttribute("value-ref");
                        String hireEmployeeDataKeyRef = hireEmployeeDataChild.getAttribute("key-ref");
                        Object valueObject = null;
                        Object keyObject = null;
                        if ((hireEmployeeDataValueRef!= null)&&(!StringUtils.isBlank(hireEmployeeDataValueRef))) {
                            valueObject = new RuntimeBeanReference(hireEmployeeDataValueRef);
                        } else {
                            valueObject = hireEmployeeDataChild.getTextContent();
                        }
                        if ((hireEmployeeDataKeyRef!= null)&&(!StringUtils.isBlank(hireEmployeeDataKeyRef))) {
                            keyObject = new RuntimeBeanReference(hireEmployeeDataKeyRef);
                        } else {
                            keyObject = hireEmployeeDataChild.getAttribute("key");
                        }
                        if ((keyObject == null)||((keyObject instanceof String)&&StringUtils.isBlank(((String) keyObject)))) {
                            keyObject = hireEmployeeDataChild.getTagName();
                        }
                        hireEmployeeData.put(keyObject, valueObject);
                    }
                }
                builder.addPropertyValue("hireEmployeeData", hireEmployeeData);
            }
        }
        if ((element.getAttribute("autoComplete")!= null)&&(!StringUtils.isBlank(element.getAttribute("autoComplete")))) {
            builder.addPropertyValue("autoComplete", element.getAttribute("autoComplete"));
        }
        if ((element.getAttribute("runNow")!= null)&&(!StringUtils.isBlank(element.getAttribute("runNow")))) {
            builder.addPropertyValue("runNow", element.getAttribute("runNow"));
        }
        Element commentDataListElement = null;
        commentDataListElement = DomUtils.getChildElementByTagName(element, "comment-data");
        List<Element> commentDataListChilds = null;
        if (commentDataListElement!= null) {
            String commentDataRef = commentDataListElement.getAttribute("ref");
            if ((commentDataRef!= null)&&(!StringUtils.isBlank(commentDataRef))) {
                if ((!commentDataRef.startsWith(patternInfo.getPrefix()))&&(!commentDataRef.endsWith(patternInfo.getSuffix()))) {
                    builder.addPropertyValue("commentData", new RuntimeBeanReference(commentDataRef));
                } else {
                    builder.addPropertyValue("commentData", commentDataRef);
                }
            } else {
                ManagedMap commentData = new ManagedMap();
                commentDataListChilds = DomUtils.getChildElementsByTagName(commentDataListElement, "comment-datum");
                if (commentDataListChilds!= null) {
                    if (commentDataListChilds.size() == 0) {
                        commentDataListChilds = DomUtils.getChildElements(commentDataListElement);
                    }
                    for (Element commentDataChild: commentDataListChilds) {
                        String commentDataValueRef = commentDataChild.getAttribute("value-ref");
                        String commentDataKeyRef = commentDataChild.getAttribute("key-ref");
                        Object valueObject = null;
                        Object keyObject = null;
                        if ((commentDataValueRef!= null)&&(!StringUtils.isBlank(commentDataValueRef))) {
                            valueObject = new RuntimeBeanReference(commentDataValueRef);
                        } else {
                            valueObject = commentDataChild.getTextContent();
                        }
                        if ((commentDataKeyRef!= null)&&(!StringUtils.isBlank(commentDataKeyRef))) {
                            keyObject = new RuntimeBeanReference(commentDataKeyRef);
                        } else {
                            keyObject = commentDataChild.getAttribute("key");
                        }
                        if ((keyObject == null)||((keyObject instanceof String)&&StringUtils.isBlank(((String) keyObject)))) {
                            keyObject = commentDataChild.getTagName();
                        }
                        commentData.put(keyObject, valueObject);
                    }
                }
                builder.addPropertyValue("commentData", commentData);
            }
        }
        BeanDefinition definition = builder.getBeanDefinition();
        definition.setAttribute(MuleHierarchicalBeanDefinitionParserDelegate.MULE_NO_RECURSE, Boolean.TRUE);
        MutablePropertyValues propertyValues = parserContent.getContainingBeanDefinition().getPropertyValues();
        if (parserContent.getContainingBeanDefinition().getBeanClassName().equals("org.mule.config.spring.factories.PollingMessageSourceFactoryBean")) {
            propertyValues.addPropertyValue("messageProcessor", definition);
        } else {
            PropertyValue messageProcessors = propertyValues.getPropertyValue("messageProcessors");
            if ((messageProcessors == null)||(messageProcessors.getValue() == null)) {
                propertyValues.addPropertyValue("messageProcessors", new ManagedList());
            }
            List listMessageProcessors = ((List) propertyValues.getPropertyValue("messageProcessors").getValue());
            listMessageProcessors.add(definition);
        }
        return definition;
    }

    protected String getAttributeValue(Element element, String attributeName) {
        if (!StringUtils.isEmpty(element.getAttribute(attributeName))) {
            return element.getAttribute(attributeName);
        }
        return null;
    }

    private String generateChildBeanName(Element element) {
        String id = SpringXMLUtils.getNameOrId(element);
        if (StringUtils.isBlank(id)) {
            String parentId = SpringXMLUtils.getNameOrId(((Element) element.getParentNode()));
            return ((("."+ parentId)+":")+ element.getLocalName());
        } else {
            return id;
        }
    }

}
